"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("../WhitePage/index");
var uuid_1 = require("../../utils/uuid");
var dom_1 = require("../../utils/dom");
require("./index.less");
var Siema = require("siema");
var prefix = 'fcw-board';
var AbstractWhiteboard = (function () {
    function AbstractWhiteboard(target, _a) {
        var _b = _a === void 0 ? {} : _a, sources = _b.sources, eventHub = _b.eventHub, visiblePageIndex = _b.visiblePageIndex, allowRollback = _b.allowRollback, onlyEmitSnap = _b.onlyEmitSnap;
        this.id = uuid_1.uuid();
        this.sources = [];
        this.imgEles = [];
        this.allowRollback = false;
        this.onlyEmitSnap = false;
        this.snapInterval = 15 * 1000;
        this.mode = 'master';
        this.isFullscreen = false;
        this.pages = [];
        this.snapHistory = [];
        this.isInitialized = false;
        this.isSyncing = false;
        this.visiblePageIndex = 0;
        if (target) {
            this.target = target;
        }
        else {
            this.target = document.createElement('div');
            document.body.appendChild(this.target);
        }
        if (!this.target.id) {
            this.target.id = this.id;
        }
        dom_1.addClassName(this.target, prefix);
        if (sources) {
            this.sources = sources;
        }
        this.eventHub = eventHub;
        if (typeof visiblePageIndex !== 'undefined') {
            this.visiblePageIndex = visiblePageIndex;
        }
        this.onlyEmitSnap = !!onlyEmitSnap;
        if (typeof allowRollback !== 'undefined') {
            this.allowRollback = !!allowRollback;
        }
        this.init();
    }
    Object.defineProperty(AbstractWhiteboard.prototype, "activePage", {
        get: function () {
            return this.pages[this.visiblePageIndex];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractWhiteboard.prototype, "pageMap", {
        get: function () {
            var map = {};
            this.pages.forEach(function (p) { return (map[p.id] = p); });
            return map;
        },
        enumerable: true,
        configurable: true
    });
    AbstractWhiteboard.prototype.open = function () {
        var _this = this;
        this.pages.forEach(function (page, i) {
            page.open();
            if (i !== _this.visiblePageIndex) {
                page.hide();
            }
        });
    };
    AbstractWhiteboard.prototype.close = function () {
        if (this.emitInterval) {
            clearInterval(this.emitInterval);
        }
    };
    AbstractWhiteboard.prototype.show = function () {
        if (this.activePage) {
            this.activePage.show();
        }
    };
    AbstractWhiteboard.prototype.hide = function () {
        if (this.activePage) {
            this.activePage.hide();
        }
    };
    AbstractWhiteboard.prototype.emit = function (borderEvent) {
        if (this.mode !== 'master' || !this.eventHub) {
            return;
        }
        if (this.onlyEmitSnap) {
            if (borderEvent.event !== 'borderSnap') {
                return;
            }
        }
        if (this.allowRollback &&
            (borderEvent.event === 'addMarker' || borderEvent.event === 'removeMarker')) {
            if (this.snapHistory.length > 20) {
                this.snapHistory.shift();
            }
            this.snapHistory.push(this.captureSnap(false));
        }
        borderEvent.timestamp = Math.floor(Date.now() / 1000);
        this.eventHub.emit('sync', borderEvent);
    };
    AbstractWhiteboard.prototype.captureSnap = function (shadow) {
        if (shadow === void 0) { shadow = true; }
        if (shadow) {
            return {
                id: this.id,
                sources: this.sources,
                pageIds: this.pages.map(function (page) { return page.id; }),
                visiblePageIndex: this.visiblePageIndex
            };
        }
        return {
            id: this.id,
            sources: this.sources,
            pageIds: this.pages.map(function (page) { return page.id; }),
            visiblePageIndex: this.visiblePageIndex,
            pages: this.pages.map(function (p) { return p.captureSnap(); })
        };
    };
    AbstractWhiteboard.prototype.rollbackSnap = function () {
        if (this.snapHistory.length === 0) {
            return;
        }
        this.snapHistory.pop();
        var snap = this.snapHistory[this.snapHistory.length - 1];
        if (snap) {
            this.applySnap(snap);
        }
    };
    AbstractWhiteboard.prototype.destroy = function () {
        if (this.emitInterval) {
            clearInterval(this.emitInterval);
        }
        if (this.eventHub) {
            this.eventHub.removeAllListeners();
        }
        if (this.siema) {
            this.siema.destroy();
        }
        this.imgsContainer.remove();
        this.pagesContainer.remove();
        this.pages.forEach(function (page) {
            page.destroy();
        });
    };
    AbstractWhiteboard.prototype.initSiema = function () {
        var _this = this;
        this.sources.forEach(function (source) {
            var imgEle = document.createElement('div');
            dom_1.addClassName(imgEle, prefix + "-img-wrapper");
            imgEle.style.backgroundImage = "url(" + source + ")";
            _this.imgEles.push(imgEle);
            _this.imgsContainer.appendChild(imgEle);
        });
        this.siema = new Siema({
            selector: this.imgsContainer,
            duration: 200,
            easing: 'ease-out',
            perPage: 1,
            startIndex: 0,
            draggable: false,
            multipleDrag: true,
            threshold: 20,
            loop: false,
            rtl: false
        });
    };
    AbstractWhiteboard.prototype.applySnap = function (snap) {
        var _this = this;
        var id = snap.id, sources = snap.sources, pageIds = snap.pageIds;
        if (!this.isInitialized && !this.isSyncing) {
            this.id = id;
            this.sources = sources;
            this.isSyncing = true;
            this.sources.forEach(function (source, i) {
                var page = new index_1.WhitePage({ imgSrc: source }, {
                    mode: _this.mode,
                    whiteboard: _this,
                    parentContainer: _this.pagesContainer
                });
                page.id = pageIds[i];
                page.container.style.visibility = 'hidden';
                _this.pages.push(page);
                page.open();
            });
            this.initSiema();
            this.isInitialized = true;
            this.isSyncing = false;
        }
        this.onPageChange(snap.visiblePageIndex);
        (snap.pages || []).forEach(function (pageSnap) {
            var page = _this.pageMap[pageSnap.id];
            if (page) {
                page.applySnap(pageSnap);
            }
        });
    };
    return AbstractWhiteboard;
}());
exports.AbstractWhiteboard = AbstractWhiteboard;
